#! /usr/bin/perl

################################################################################
#                         HDL_DUMB GUI FOR LINUX                               #
#                                                                              #
#                      Copyright 2007 Caleb Griffith                           #
################################################################################
# GUI for HDL_DUMP a utility for sending playstation 2 games to a hard drive   #
# Intended to mimic the look and feel of the official hdl_dumb for windows.    #
################################################################################

require Tk;
use Tk;
use Tk::JComboBox;
use Tk::ProgressBar;
use Tk::Labelframe;
use Tk::Frame;
use Tk::Entry;
use Tk::Label;
use Tk::Radiobutton;
use Tk::Checkbutton;
use Tk::Button;
use Tk::Bitmap;
use Tk::Photo;
use Tk::HList;

$font = 'monospace';


# Create the GUI and all objects that reside in the main window
&Define_MainWindow;
&Define_Target_Frame;
&Define_Operation_Frame;
$Install_Frame = $main->Frame(-height=>15)->pack(-fill=>'y',-fill=>'x');
&Define_Source_Frame;
&Define_Copy_Frame;
&Define_Game_Frame;
&Define_Browse_Frame;
&Define_Button_Frame;
&Define_Copy_Button_Frame;
&Define_Delete_Button_Frame;

# If config file exists...
if(-e "hdl_conf")
{
    # Read i
    &ReadConfig();
}
else
{
    # Else create it
    &CreateConfig();
}

# Set default values
$Location_Value='1';
$Operation_Value='0';
$Source_Value='0';
$Type_Cbo->setSelectedIndex(0);

MainLoop();
################################################################################
################################################################################


################################################################################
sub Define_MainWindow
# Creates the mainwindow and all it's proprties, also instantiates the icons
# For the Browse window, since they are called against the main window
################################################################################
{
    $main = MainWindow->new();
    
    $icon = $main->Photo(-file => "pix/icon.xpm", -format => 'xpm'); 
    $main->Icon(-image => $icon);
    
    $main->OnDestroy(\&CleanUp);
    $main->geometry('=400x378');
    $main->minsize( qw(400 378) );
    $main->maxsize( qw(400 378) );
    
    $main->optionAdd("*font"=>"$font 10");
    
    $main->title("hdl_dumb_linux");
    
    $dvdimage = $main->Photo(-file => 'pix/DVD.gif');
    $dvddlimage = $main->Photo(-file => 'pix/DVDDL.gif');
    $cdimage = $main->Photo(-file => 'pix/CD.gif');
}

################################################################################
sub Define_Target_Frame
# Creates the target frame and all of it's inherent widgets
# This is where the user selects their local HD or a networked PS2 Device
################################################################################
{
    $Target_Frame = $main->Labelframe
    (
        -text=>'Target',
        -font=>"$font 10",
        -relief=>'sunken',
        -borderwidth=>1
    )->pack(-side=>'top',-fill=>'x');
    
    $Drive_Frame = $Target_Frame->Frame()->pack(-side=>'top',-fill=>'x');
    $Drive_Frame->Radiobutton
    (
#        -state=>'disabled',
        -command=>\&Drive_Radio_Press,
        -font=>"$font 10",
        -text=>'Locally connected Playstation 2 HDD',
        -value=>'0',
        -variable=>\$Location_Value                                    
    )->pack(-side=>'left',-anchor=>'w');
    $Drive_Cbo = $Drive_Frame->JComboBox
    (
        -state=>'disabled',
        -disabledbackground=>'gray',
        -font=>"$font 10",
        -textvariable => \$Local_Value,
        -entrywidth=>10,
        -listwidth=>10,
        -background=>'white',
        -borderwidth=>1,
        -choices =>
        [
            {-name => " ", -value => '0'}
        ]
    )->pack(-side=>'right');
    
    $Network_Frame = $Target_Frame->Frame()->pack(-side=>'top',-fill=>'x');
    $Network_Radio = $Network_Frame->Radiobutton(
                    -text=>'Networking server, at ',
                    -command=>\&Network_Radio_Press,
                    -font=>"$font 10",
                    -value=>'1',
                    -variable=>\$Location_Value
                    )->pack(-side=>'left',-anchor=>'w');
    $Network_Text = $Network_Frame->Entry
    (
        -text=>"",
        -font=>"$font 10 bold",
        -background=>'white',
        -width=>18
    )->pack(-sid=>'left');
    $Network_Text->bind('<FocusOut>' => sub {$ps2_ip = $Network_Text->get;}); 
    $Network_Button = $Network_Frame->Button
    (
        -text=>'Attach',
        -font=>"$font 10",
        -command=>\&Attach
    )->pack(-side=>'right',-padx=>5);
    
    $Space_Frame = $Target_Frame->Frame()->pack(-side=>'top',-fill=>'x');
    $Space_Frame->Label
    (
        -text=>'Disk space: ',
        -font=>"$font 10"
    )->pack(-side=>'left',-anchor=>'w');
    
    $Space_Progress = $Space_Frame->ProgressBar
    (
        -from=>0,
        -to=>300,
        -blocks=>1,
        -colors => [0, 'green', 80, 'yellow' , 90, 'red']
    )->pack(-side=>'left');
    $Space_Lbl_Frame = $Target_Frame->Frame()->pack(-side=>'bottom',-fill=>'x');
    $Space_Lbl_Size = $Space_Lbl_Frame->Label
    (
        -text=>"",
        -font=>"$font 10"
    )->pack(-side=>'right');
}

################################################################################
sub Define_Operation_Frame
# Creates the operation frame and all of it's inherent widgets
# This is where the user selects to install a new game
# Or browse/delete what is already on his device.
################################################################################
{
    $Operation_Frame = $main->Frame()->pack(-side=>'top',-fill=>'x');
    $Operation_Frame->Label
    (
        -text=>'Operation:  ',
        -font=>"$font 10"
    )->pack(-side=>'left');
    $Install_Radio = $Operation_Frame->Radiobutton
    (
        -text=>'Install games',
        -font=>"$font 10",
        -value=>'0',
        -command=>\&Install_Radio,
        -variable=>\$Operation_Value
    )->pack(-side=>'left');
    $Browse_Radio = $Operation_Frame->Radiobutton
    (
        -text=>'Browse and delete',
        -font=>"$font 10",
        -value=>'1',
        -command=>\&Browse_Radio,
        -variable=>\$Operation_Value
    )->pack(-side=>'right');
}

################################################################################
sub Define_Source_Frame
# Creates the Source frame and all of it's inherent widgets
# This is where the user selects what the source device of the install is
# This can either be an ISO image or a local cdrom device.
################################################################################
{
    $Source_Frame = $Install_Frame->Labelframe
    (
        -text=>'Source',
        -font=>"$font 10",
        -relief=>'sunken',
        -borderwidth=>1
    )->pack(-side=>'top',-fill=>'x');
    $Iso_Frame = $Source_Frame->Frame()->pack(-side=>'top',-fill=>'x');
    $Iso_Frame->Radiobutton
    (
        -text=>'ISO file  ',
        -command=>\&Iso_Radio_Press,
        -font=>"$font 10",
        -value=>'0',
        -variable=>\$Source_Value
    )->pack(-side=>'left');
    $Iso_Entry = $Iso_Frame->Entry
    (
        -width=>35,
        -state=>'readonly',
        -font=>"$font 10",
        -readonlybackground=>'white'
    )->pack(-side=>'left');
    $Iso_Button = $Iso_Frame->Button
    (
        -text=>'...',
        -font=>"$font 10",
        -width=>3,
        -command=>\&Iso_Select
    )->pack(-side=>'right',-padx=>5);
    $Source_Frame->Frame()->pack(-side=>'top',-fill=>'x');
    $Source_Frame->Radiobutton
    (
        -text=>'Optical drive   ',
        -font=>"$font 10",
        -value=>'1',
        -command=>\&Optical_Radio_Press,
        -variable=>\$Source_Value
    )->pack(-side=>'left');
    $OpticalDrive_Cbo = $Source_Frame->JComboBox
    (
        -textvariable => \$OpticalDrive_Value,
        -state=>'disabled',
        -entrywidth=>10,
        -listwidth=>10,
        -font=>"$font 10",
        -background=>'white',
        -borderwidth=>1,
        -choices =>
        [
            {-name => ' ', -value => '0'},
            {-name => 'cdrom', -value => '1'}
        ],
        -selectcommand=>\&OpticalDrive_Cbo_Press
    )->pack(-side=>'left');   
}

################################################################################
sub Define_Game_Frame
# Creates the Game frame and all of it's inherent widgets
# This is where the user selects the title for the game, the startup
# Executable, if the source disk is a CDROM or DVDROM, and any flags they wish.
################################################################################
{
    $Game_Frame = $Install_Frame->Labelframe
    (
        -text=>'Game',
        -font=>"$font 10",
        -relief=>'sunken',
        -borderwidth=>1
    )->pack(-side=>'top',-fill=>'x');
    $Type_Frame = $Game_Frame->Frame()->pack(-side=>'top',-fill=>'x');
    $Type_Frame->Label
    (
        -text=>'Type         ',
        -font=>"$font 10"
    )->pack(-side=>'left');
    $Type_Cbo = $Type_Frame->JComboBox
    (
        -textvariable => \$Type_Value,
        -entrywidth=>10,
        -listwidth=>10,
        -font=>"$font 10",
        -background=>'white',
        -borderwidth=>1,
        -choices =>
        [
            {-name => 'DVD-ROM', -value => 0},
            {-name => 'CD-ROM', -value => 1}
        ]
    )->pack(-side=>'left');
    $Size_Lbl = $Type_Frame->Label
    (
        -text=>"",
        -font=>"$font 8"
    )->pack(-side=>'left');
    $Name_Frame = $Game_Frame->Frame()->pack(-side=>'top',-fill=>'x');
    $Name_Frame->Label
    (
        -text=>'Name:        ',
        -font=>"$font 10"
    )->pack(-side=>'left');
    $Name_Entry = $Name_Frame->Entry
    (
        -width=>35,
        -font=>"$font 10",
        -background=>'white'
    )->pack(-side=>'left');
    $Startup_Frame = $Game_Frame->Frame()->pack(-side=>'top',-fill=>'x');
    $Startup_Frame->Label
    (
        -text=>'Startup:     ',
        -font=>"$font 10"
    )->pack(-side=>'left');
    $Startup_Entry = $Startup_Frame->Entry
    (
        -width=>14,
        -font=>"$font 10",
        -background=>'white'
    )->pack(-side=>'left');
    $FlagHolder_Frame = $Game_Frame->Frame()->pack(-side=>'top',-fill=>'x');
    $Compatibility_Frame = $FlagHolder_Frame->Frame()->pack(-side=>'left', -fill=>'y');
    $Compatibility_Frame->Label
    (
        -text=>'Compatibility: ',
        -font=>"$font 10"
    )->pack(-side=>'top',-anchor=>'n');
    $Flag1_Frame = $FlagHolder_Frame->Frame()->pack(-side=>'left',);
    $ChkBox1 = $Flag1_Frame->Checkbutton
    (
        -variable=>\$ChkBox1_Value,
        -font=>"$font 8",
        -onvalue=>'+1',
        -offvalue=>'',
        -text=>'1: Slow HDD access'
    )->pack(-side=>'top',-anchor=>'w');
    $ChkBox2 = $Flag1_Frame->Checkbutton
    (
        -variable=>\$ChkBox2_Value,
        -font=>"$font 8",
        -onvalue=>'+2',
        -offvalue=>'',
        -text=>'2: No DVD9 support'
    )->pack(-side=>'top',-anchor=>'w');
    $ChkBox3 = $Flag1_Frame->Checkbutton
    (
        -variable=>\$ChkBox3_Value,
        -font=>"$font 8",
        -onvalue=>'+3',
        -offvalue=>'',
        -text=>'3: Kill HD Loader'
    )->pack(-side=>'top',-anchor=>'w');
    $ChkBox4 = $Flag1_Frame->Checkbutton
    (
        -variable=>\$ChkBox4_Value,
        -font=>"$font 8",
        -onvalue=>'+4',
        -offvalue=>'',
        -text=>'4: Date/time fix 2'
    )->pack(-side=>'top',-anchor=>'w');
    
    $Flag2_Frame = $FlagHolder_Frame->Frame()->pack(-side=>'right',);
    $ChkBox5 = $Flag2_Frame->Checkbutton
    (
        -variable=>\$ChkBox5_Value,
        -font=>"$font 8",
        -onvalue=>'+5',
        -offvalue=>'',
        -text=>'5: Date/time fix 3'
    )->pack(-side=>'top',-anchor=>'w');
    $ChkBox6 = $Flag2_Frame->Checkbutton
    (
        -variable=>\$ChkBox6_Value,
        -font=>"$font 8",
        -onvalue=>'+6',
        -offvalue=>'',
        -text=>'6: No network support'
    )->pack(-side=>'top',-anchor=>'w');
    $ChkBox7 = $Flag2_Frame->Checkbutton
    (
        -variable=>\$ChkBox7_Value,
        -font=>"$font 8",
        -onvalue=>'+7',
        -offvalue=>'',
        -text=>'7: No UAB patch'
    )->pack(-side=>'top',-anchor=>'w');
    $ChkBox8 = $Flag2_Frame->Checkbutton
    (
        -variable=>\$ChkBox8_Value,
        -font=>"$font 8",
        -onvalue=>'+8',
        -offvalue=>'',
        -text=>'8: Soul Calibur 2'
    )->pack(-side=>'top',-anchor=>'w');
}

################################################################################
sub Define_Button_Frame
# Creates the button frame and all of it's inherent widgets
# This is where the user selects if they want to Install or Close the app.
################################################################################
{
    $Button_Frame = $main->Frame()->pack(-side=>'bottom',-fill=>'y',-fill=>'x');
    $Button_Frame->Button
    (
        -text=>'Close',
        -font=>"$font 10",
        -width=>10,
        -command=> sub {Tk::exit}
    )->pack(-side=>'right',-anchor=>'s',-pady=>0,-padx=>2);
    $Button_Frame->Button
    (
        -text=>'Install',
        -font=>"$font 10",
        -width=>10,
        -command=>\&Install
    )->pack(-side=>'right',-anchor=>'s',-pady=>0,-padx=>2);   
}

################################################################################
sub Define_Delete_Button_Frame
# Creates the delete button frame and all of it's inherent widgets
# This is where the user selects if they want to Delete an image or 
# Close the app.
#
# Hidden by default
################################################################################
{
    $Delete_Button_Frame = $main->Frame()->pack(-side=>'bottom',-fill=>'y',-fill=>'x');
    $Delete_Button_Frame->Button
    (
        -text=>'Close',
        -font=>"$font 10",
        -width=>10,
        -command=> sub {Tk::exit}
    )->pack(-side=>'right',-anchor=>'s',-pady=>0,-padx=>2);
    $Delete_Button_Frame->Button
    (
        -text=>'Delete',
        -font=>"$font 10",
        -width=>10,
        -command=>\&Delete
    )->pack(-side=>'right',-anchor=>'s',-pady=>0,-padx=>2);
    
    $Delete_Button_Frame->pack('forget');
}

################################################################################
sub Define_Copy_Button_Frame
# Creates the copy button frame and all of it's inherent widgets
# This is where the user can cancel the install if the install is already
# in Progress
#
# Hidden by default
################################################################################
{
    $Copy_Button_Frame = $main->Frame()->pack(-side=>'bottom',-fill=>'y',-fill=>'x');
    $Copy_Button_Frame->Button
    (
        -text=>'Cancel',
        -font=>"$font 10",
        -width=>10,
        -command=>\&Cancel
    )->pack(-side=>'right',-anchor=>'s',-pady=>0,-padx=>2);
    
    $Copy_Button_Frame->pack('forget');
}

################################################################################
sub Define_Browse_Frame
# Creates the browse frame and all of it's inherent widgets
# This is where the user can see what games are installed on his device, and
# Delete them if he wishes
#
# Hidden by default
################################################################################
{
    $Browse_Frame = $main->Frame()->pack(-fill=>'y',-fill=>'x');
    $Box_Frame = $Browse_Frame->Frame()->pack(-fill=>'x');

    $Listbox_Frame = $Box_Frame->Frame()->pack(-side=>'top');
    
    $Browse_List = $Listbox_Frame->Scrolled
    (
        "HList",
        -header => 1,
        -columns => 3,
        -font=>"$font 10",
        -scrollbars => 'e',
        -width => 55,
        -height => 15,
        -selectmode => 'single',
        -background => 'white'
    )->pack(-fill => 'both'); 
    $Browse_List->header('create', 0, -text => 'Partition Information   ');
    $Browse_List->header('create', 1, -text => '          Flags');
    $Browse_List->header('create', 2, -text => '      Size'); 

    $Browse_Frame->pack('forget');
}
################################################################################
sub Define_Copy_Frame
# Creates the copy frame and all of it's inherent widgets
# This is where progress will be shown when a game is being transfered to the 
# Device.
#
# Hidden by default
################################################################################
{
    $Copy_Frame = $Install_Frame->Frame()->pack(-side=>'top',-fill=>'y',-fill=>'x');

    $Progress_Frame = $Copy_Frame->Labelframe
    (
        -text=>'Installing',
        -font=>"$font 10",
        -relief=>'groove',
        -borderwidth=>1
    )->pack(-side=>'top',-fill=>'x');
                
    $Progress_Label = $Progress_Frame->Label
    (
    	-text=>"",   
    )->pack(-side=>'top');
    $Install_Progress = $Progress_Frame->ProgressBar
    (
        -from=>0,
        -to=>350,
        -blocks=>1,
        -colors => [0, 'blue', 50, 'blue' , 80, 'blue'],
    )->pack(-side=>'top');
    $Install_Label = $Progress_Frame->Label
    (
    	-text=>"",   
    )->pack(-side=>'bottom');
    
    $Copy_Frame->pack('forget');
}

###############################################################################
#                         END WINDOW DEFINATIONS                              #
###############################################################################

################################################################################
sub Attach
# onclick action of the Attach_Button
#
# Connects to the selected device and reads the Table of Contents from the
# Drive. Then displays the amount of space used and left in a progress bar 
# aswell in a label.
################################################################################
{
    @GameList = ();
	
    # Determine how
    if($Location_Value == 0)
    {
        $cmd = "gksudo $hdl_dump hdl_toc /dev/$Local_Value";
    }
    elsif($Location_Value == 1)
    {
        $Network_Value = $Network_Text->get;
        
        $cmd = "$hdl_dump hdl_toc $Network_Value";
    }
    
    # Read TOC
    @Result = `$cmd 2>&1`;
    
    $LastLine = undef;
    # Parse result
    foreach $result (@Result)
    {
    	chomp($result);
    	
        if($result =~ m/No route to host/)
        {
            $main->messageBox
            (
                -type=>"ok",
                -message=>"Couldn't connect to PS2",
                -title=>'Error',
                -font=>"$font 10"
            );
            $LastLine = undef;
        }
        else
        {
            $hdl_type = substr($result,0,3);
            $hdl_size = substr($result,4,9);
            $hdl_flags = substr($result,13,16);
            $hdl_name = substr($result,43,50);
            if( ($hdl_type eq 'DVD') || ($hdl_type eq 'CD '))
            {
                $hdl_size =~ s/KB//;
                $hdl_size = $hdl_size / 1024;
                $hdl_size = int($hdl_size + .5);
                
                $hdl_type =~ s/^\s+//;
                $hdl_type =~ s/\s+$//;
                $hdl_flags =~ s/^\s+//;
                $hdl_flags =~ s/\s+$//;
                $hdl_name =~ s/^\s+//;
                $hdl_name =~ s/\s+$//;
                
                if($hdl_size > 4812)
                {
                    $hdl_type = 'DVDDL';
                }
                
                push(@GameList,"$hdl_type|$hdl_size|$hdl_flags|$hdl_name")
            }
            elsif($hdl_type eq 'tot')
            {
                $LastLine = $result;
            }
        }
    }
    
    # No Error
    unless ($LastLine eq undef)
    {
    	# Update ProgressBar and Drive size label
        ($Filler,$Disk_Total,$Filler,$Disk_Used,$Filler,$Disk_Available) = split(/\s/,$LastLine);
        
        $Disk_Used =~ s/MB,//;
        $Disk_Available =~ s/MB//;
        $Disk_Total =~ s/MB,//;
        
        $Space_Progress->configure(-to=>$Disk_Total);
        $Space_Progress->value($Disk_Used);
        $Space_Lbl_Size->configure(-text=>"$Disk_Available MB of $Disk_Total MB free");
    	
        $Browse_List->delete('all');
        &PopulateBrowseBox;
    }
 }

#################################################################################
sub Install_Radio
# onclick action of the Install_Radiobutton
#
# Hides the browse_frame and the delete_button_frame, and shows the 
# install_frame and the button_frame.
#################################################################################
{
    $Browse_Frame->pack('forget');
    $Delete_Button_Frame->pack('forget');
    
    $Install_Frame->pack(-fill=>'y',-fill=>'x');
    $Button_Frame->pack(-side=>'bottom',-fill=>'y',-fill=>'x');
}

#################################################################################
sub Browse_Radio
# onclick action of the Browse_Radiobutton
#
# Hides the install_frame and the button_frame, and shows the 
# browse_frame and the delete_button_frame.
#################################################################################
{
    $Install_Frame->pack('forget');
    $Button_Frame->pack('forget');

    $Browse_Frame->pack(-fill=>'y',-fill=>'x');
    $Delete_Button_Frame->pack(-side=>'bottom',-fill=>'y',-fill=>'x');
    
    $Browse_List->delete('all');
    &PopulateBrowseBox;
}

#################################################################################
sub Iso_Select
# onclick action of the Iso_Button
#
# Opens a file select dialog box, and once a file is selected, places that file
# name and path into the Iso_Entry field.
#################################################################################
{
    my $types =
    [
        ['ISO images',['.iso','.ISO']],
     	['All Files','*',],
    ];
    
    $Iso_File = $main->getOpenFile
    (
    	-initialdir => '~',
    	-defaultextension => '.iso',
        -filetypes => $types
    );
    $Iso_Entry->configure(-text => $Iso_File);
    
    $Iso_File =~ s/ /\\ /g;
    
    $Startup_Entry->delete(0,length($Startup_Entry->get));
    $Name_Entry->delete(0,length($Name_Entry->get));
    $Size_Lbl->configure(-text=>"");
    
    if(uc($Iso_File) =~ m/.ISO/)
    {
        $cmd = "$hdl_dump cdvd_info $Iso_File";
        
        &ReadDiskInfo;
    }
    $Iso_Entry->xviewScroll(2,'pages');
}

#################################################################################
sub Delete
# onclick action of the Delete_Button
#
# Reads the currently selected item in browse_list, and prompts to delete that
# item from the connected device.
#################################################################################
{
    # Determine how
    if($Location_Value == 0)
    {
        $cmd = "gksudo $hdl_dump delete /dev/$Local_Value";
    }
    elsif($Location_Value ==1)
    {
        $Network_Value = $Network_Text->get;
        
        $cmd = "$hdl_dump delete $Network_Value";
    }
    
    @SelectedGame = $Browse_List->selectionGet();
    
    unless($SelectedGame[0] eq undef)
    {
        $Game2Delete = $SelectedGame[0];
        $Game2Delete =~ s/-/_/g;
        
    	$cmd = $cmd . ' "PP.HDL.'.$Game2Delete.'"';
    	
    	$Delete_Result = $main->messageBox
        (
            -type=>"yesno",
            -message=>"Really delete \"$SelectedGame[0]\"?",
            -title=>'Delete',
            -font=>"$font 10"
        );
        if($Delete_Result eq 'Yes')
        {
            `$cmd`;
            &Attach;
            $Browse_List->delete('all');
            &PopulateBrowseBox;
        }	
    }
}

################################################################################
sub PopulateBrowseBox
#
# Takes all games found when attach was ran, and places them into the 
# Browse_list object.
################################################################################
{
    foreach $list (@GameList)
    {
        ($type,$size,$flags,$game) = split(/\|/,$list);
        until(length($flags) == 15)
        {
            $flags = " " . $flags;
        }
        until(length($size) == 5)
        {
            $size = " " . $size;
        }
        
        $Browse_List->add($game, -text => "stam", -data => [$game]);
        if($type eq 'DVD')
        {
            $Browse_List->itemCreate($game, 0, -text => $game, -image => $dvdimage, -itemtype => "imagetext");
        }
        elsif($type eq 'CD')
        {
            $Browse_List->itemCreate($game, 0, -text => $game, -image => $cdimage, -itemtype => "imagetext");
        }
        else
        {
            $Browse_List->itemCreate($game, 0, -text => $game, -image => $dvddlimage, -itemtype => "imagetext");
        }
        $Browse_List->itemCreate($game, 1, -text => $flags);
        $Browse_List->itemCreate($game, 2, -text => "$size MB"); 
    }
}

################################################################################
sub Optical_Radio_Press
# onclick action of the Optical_Radiobutton
#
# Disables and emptys the Iso_Entry and Iso_Button widgets,
# Enables the OpticalDrive_combobox.
################################################################################
{
    $Iso_Entry->configure(-state=>'disabled');
    $Iso_Button->configure(-state=>'disabled');
    $OpticalDrive_Cbo->configure(-state=>'normal');
    $Iso_Entry->delete(0,length($Iso_Entry->get));
    
    $Startup_Entry->delete(0,length($Startup_Entry->get));
    $Name_Entry->delete(0,length($Name_Entry->get));
    $Size_Lbl->configure(-text=>"");
}

################################################################################
sub Iso_Radio_Press
# onclick action of the Iso_Radiobutton
#
# Disables the OpticalDrive_combobox,
# Enables the Iso_Entry and Iso_Button widgets.
################################################################################
{
    $Iso_Entry->configure(-state=>'readonly');
    $Iso_Button->configure(-state=>'normal');
    $Startup_Entry->delete(0,length($Startup_Entry->get));
    $Name_Entry->delete(0,length($Name_Entry->get));
    $OpticalDrive_Cbo->configure(-state=>'disabled');
    $OpticalDrive_Cbo->setSelectedIndex(0);
}

################################################################################
sub Drive_Radio_Press
# onclick action of the Drive_Radiobutton
#
# Disables the Network_Entry,
# Enables the Drive_ComboBox.
################################################################################
{
    $Network_Text->configure(-state=>'disabled');
    $Drive_Cbo->configure(-state=>'normal');
    $Drive_Cbo->setSelectedIndex(0);
    
    if($FindDrives eq undef)
    {
        &QueryLocalDrives;
    }
}

################################################################################
sub QueryLocalDrives
#
# Finds any and all locally connected PS2 hard drives.
################################################################################
{
    $AllowDriveAccess = $main->messageBox
        (
	    -type=>"yesno",
	    -message=>"Attempting to query locally connected PS2 Hard Drives.\nThis requires root access. Allow access?",
	    -title=>'Allow access',
            -font=>"$font 10"
    	);
    if($AllowDriveAccess eq 'Yes')
    {
        opendir(DEVICES, "/dev") || die "Cannot opendir /dev: $!";
        foreach $device (sort readdir(DEVICES))
        {
            if( ($device =~ m/^hd/) || ($device =~ m/^sd/) )
            {
                $toc = `gksudo ./hdl_dump hdl_toc /dev/$device`;
                if($toc =~ m/^type/)
                {
                    push(@DriveChoice,$device);
                }
            }
        }
        closedir(DEVICES);
        
        if($DriveChoice[0] eq undef)
        {
            $main->messageBox
            (
                -type=>"ok",
                -message=>"No local PS2 formatted drives found.\nVerify your drive is connected and try again.",
                -title=>'Error',
                -font=>"$font 10"
            );
            $Network_Radio->select;
            &Network_Radio_Press;
        }
        else
        {
            $FindDrives = 'found';
            foreach $drive (@DriveChoice)
            {
                $Drive_Cbo->addItem($drive, -value => $drive);
            }
        }
    }
    else
    {
        $Network_Radio->select;
        &Network_Radio_Press;
    }
}

################################################################################
sub Network_Radio_Press
# onclick action of the Network_Radiobutton
#
# Disables the Drive_ComboBox,
# Enables the Network_Entry.
################################################################################
{
    $Network_Text->configure(-state=>'normal');
    $Drive_Cbo->configure(-state=>'disabled');
    $Drive_Cbo->setSelectedIndex(0);
}

################################################################################
sub OpticalDrive_Cbo_Press
# onselect action of the OpticalDrive_combobox
#
# If cdrom is selected, read the device, and populate startup, name, and size
# Widgets.
################################################################################
{
    if($OpticalDrive_Value == 1)
    {
        $cmd = "$hdl_dump cdvd_info /dev/cdrom";
        &ReadDiskInfo;		
    }
    else
    {
    
	$Startup_Entry->delete(0,length($Startup_Entry->get));
	$Name_Entry->delete(0,length($Name_Entry->get));
	$Size_Lbl->configure(-text=>"");
    }
}

################################################################################
sub ReadDiskInfo
#
# Reads the selected cdrom or ISO file, and retrevies it's startup executable,
# name, and size.
################################################################################
{
    # Read Disk Info
    $Result = `$cmd 2>&1`;
    
    chomp($Result);
    
    if($Result =~ m/No medium found/)
    {
	$main->messageBox
        (
	    -type=>"ok",
	    -message=>"No disk in drive.",
	    -title=>'Error',
	    -font=>"$font 10"
	);
	$OpticalDrive_Cbo->setSelectedIndex(0);
	$LastLine = undef;
    }
    elsif($Result =~ m/No such file or directory/)
    {
	$main->messageBox
        (
	    -type=>"ok",
	    -message=>"File couldn't be read.",
	    -title=>'Error',
	    -font=>"$font 10"
        );
        $OpticalDrive_Cbo->setSelectedIndex(0);
	$LastLine = undef;
    }
    elsif($Result =~ m/not a Playstation CD-ROM\/DVD-ROM/)
    {
	$main->messageBox
        (
	    -type=>"ok",
	    -message=>"Not a Playstation disk.",
	    -title=>'Error',
	    -font=>"$font 10"
        );
        $OpticalDrive_Cbo->setSelectedIndex(0);
	$LastLine = undef;
    }
    else
    {
        if($Result =~ m/dual layer/)
        {
            ($Disk_Exe,$Disk_Name,$FILLER,$FILLER,$Disk_Size) = split(/\s/,$Result);
        }
        else
        {
            ($Disk_Exe,$Disk_Name,$FILLER,$Disk_Size) = split(/\s/,$Result);
        }
        
        undef($FILLER);
        $Disk_Exe =~ s/\"//g;
        $Disk_Name =~ s/\"//g;
        $Disk_Size =~ s/KB//;
        $Disk_Size = $Disk_Size / 1024;
        $Disk_Size = int($Disk_Size + .5);
        
        $Aprox_Size1 = $Disk_Size / 1024;
        $Aprox_Size = int($Aprox_Size1 + .5);
        if($Aprox_Size1 > $Aprox_Size)
        {
            $Aprox_Size++;
        }
        undef($Aprox_Size1);
        $Aprox_Size = $Aprox_Size * 1024;
        
        if($Disk_Size <= 700)
        {
            $Type_Cbo->setSelectedIndex(1);
        }
        else
        {
            $Type_Cbo->setSelectedIndex(0);
        }
        
        $Startup_Entry->configure(-text=>$Disk_Exe);
        $Name_Entry->configure(-text=>$Disk_Name);
        $Size_Lbl->configure
        (
            -text=>"  $Disk_Size MB, aprox. $Aprox_Size MB reqd."
        );
    } 

}

################################################################################
sub VerifyData
#
# Verifies all required fields have been populated.
################################################################################
{
    if( ($Local_Value eq undef) && ($Network_Value eq undef) )
    {
        $main->messageBox
        (
	    -type=>"ok",
	    -message=>"Please attach a target location.",
	    -title=>'Missing required values',
	    -font=>"$font 10"
	);
        return "no";
    }
    elsif( ($Source_Value == 0) && ($Iso_Entry->get eq undef) )
    {
        $main->messageBox
        (
	    -type=>"ok",
	    -message=>"Please select a source.",
	    -title=>'Missing required values',
	    -font=>"$font 10"
	);
        return "no";
    }
    elsif($Name_Entry->get eq undef)
    {
        $main->messageBox
        (
	    -type=>"ok",
	    -message=>"A name for the game must be provided",
	    -title=>'Missing required values',
	    -font=>"$font 10"
	);
        return "no";
    }
    elsif($Startup_Entry->get eq undef)
    {
        $main->messageBox
        (
	    -type=>"ok",
	    -message=>"A startup elf name for the game must be provided",
	    -title=>'Missing required values',
	    -font=>"$font 10"
	);
        return "no";
    }
    else
    {
        return "yes";
    }
}

################################################################################
sub Install
# onclick action of the Install_Button
#
# Disables everything in game_frame, hides source_frame, shows copy_frame.
# Performs the doinstall function, then returns everything to normal.
################################################################################
{
    # Verify all fields have been populated
    $Okay2Install = &VerifyData;
    
    if($Okay2Install eq 'yes')
    {  
        $Source_Frame->pack('forget');
        $Game_Frame->pack('forget');
        $Button_Frame->pack('forget');
        $Copy_Frame->pack(-side=>'top',-fill=>'y',-fill=>'x');
        $Game_Frame->pack(-side=>'top',-fill=>'y',-fill=>'x');
        $Copy_Button_Frame->pack(-side=>'bottom',-fill=>'y',-fill=>'x');
        
        $Network_Radio->configure(-state=>'disabled');
        $Network_Text->configure(-state=>'disabled');
        $Network_Button->configure(-state=>'disabled');
        $Install_Radio->configure(-state=>'disabled');
        $Browse_Radio->configure(-state=>'disabled');
        $Type_Cbo->configure(-state=>'disabled');
        $Name_Entry->configure(-state=>'disabled');
        $Startup_Entry->configure(-state=>'disabled');
        $ChkBox1->configure(-state=>'disabled');
        $ChkBox2->configure(-state=>'disabled');
        $ChkBox3->configure(-state=>'disabled');
        $ChkBox4->configure(-state=>'disabled');
        $ChkBox5->configure(-state=>'disabled');
        $ChkBox6->configure(-state=>'disabled');
        $ChkBox7->configure(-state=>'disabled');
        $ChkBox8->configure(-state=>'disabled');
        
        
        $Progress_Label->configure(-text=> 'Installing: '.$Name_Entry->get());
        
        $main->update;
        
        $Cancel_Flag = 0;
        
        &DoInstall;
        
        if($Cancel_Flag == 0)
        {	
            $main->messageBox
            (
                -type=>"ok",
                -message=>"Installation of \"$Name\"\nis complete",
                -title=>'Installation Complete',
                -font=>"$font 10"
            );
        }	
        else
        {
            $main->messageBox
            (
                -type=>"ok",
                -message=>"Installation of \"$Name\" was Canceled",
                -title=>'Installation Canceled',
                -font=>"$font 10"
            );
        }
    
        $ChkBox1_Value = " ";
        $ChkBox2_Value = " ";
        $ChkBox3_Value = " ";
        $ChkBox4_Value = " ";
        $ChkBox5_Value = " ";
        $ChkBox6_Value = " ";
        $ChkBox7_Value = " ";
        $ChkBox8_Value = " ";
        
        $Network_Radio->configure(-state=>'normal');
        $Network_Text->configure(-state=>'normal');
        $Network_Button->configure(-state=>'normal');
        $Install_Radio->configure(-state=>'normal');
        $Browse_Radio->configure(-state=>'normal');
        $Type_Cbo->configure(-state=>'normal');
        $Name_Entry->configure(-state=>'normal');
        $Startup_Entry->configure(-state=>'normal');
        $ChkBox1->configure(-state=>'normal');
        $ChkBox2->configure(-state=>'normal');
        $ChkBox3->configure(-state=>'normal');
        $ChkBox4->configure(-state=>'normal');
        $ChkBox5->configure(-state=>'normal');
        $ChkBox6->configure(-state=>'normal');
        $ChkBox7->configure(-state=>'normal');
        $ChkBox8->configure(-state=>'normal');
        
        $Type_Cbo->setSelectedIndex(0);
        $OpticalDrive_Cbo->setSelectedIndex(0);
        $Name_Entry->delete(0,length($Name_Entry->get));
        $Size_Lbl->configure(-text=>"");
        
        $Iso_Entry->delete(0,length($Iso_Entry->get));
        $Startup_Entry->delete(0,length($Startup_Entry->get));
        $Name_Entry->delete(0,length($Name_Entry->get));
        
        $Copy_Frame->pack('forget');
        $Game_Frame->pack('forget');
        $Copy_Button_Frame->pack('forget');
        $Source_Frame->pack(-fill=>'y',-fill=>'x');
        $Game_Frame->pack(-side=>'top',-fill=>'y',-fill=>'x');
        $Button_Frame->pack(-side=>'bottom',-fill=>'y',-fill=>'x');
    }
}

################################################################################
sub DoInstall
#
# Installs the game, updating the progress bar, and progress label.
################################################################################
{
    if($Type_Value == 0){$DiskType = 'inject_dvd';}
    if($Type_Value == 1){$DiskType = 'inject_cd';}
    if($Source_Value == 0)
    {
        $SourceType = $Iso_Entry->get;
        $SourceType =~ s/ /\\ /g;
    }
    if($Source_Value == 1){$SourceType = "/dev/cdrom"}
    $Name = $Name_Entry->get();
    $Startup = $Startup_Entry->get();
    
    if($Location_Value == 0)
    {
        $cmd =
        (
        'gksudo '.$hdl_dump.' '.$DiskType.' /dev/'.$Local_Value.' "'.$Name.'" '.$SourceType.' '
        .$Startup.' '.$ChkBox1_Value.$ChkBox2_Value.$ChkBox3_Value
        .$ChkBox4_Value.$ChkBox5_Value.$ChkBox6_Value.$ChkBox7_Value
        .$ChkBox8_Value
        );
    }
    elsif($Location_Value == 1)
    {
        $Network_Value = $Network_Text->get;
        
        $cmd =
        (
        $hdl_dump.' '.$DiskType.' '.$Network_Value.' "'.$Name.'" '.$SourceType.' '
        .$Startup.' '.$ChkBox1_Value.$ChkBox2_Value.$ChkBox3_Value
        .$ChkBox4_Value.$ChkBox5_Value.$ChkBox6_Value.$ChkBox7_Value
        .$ChkBox8_Value
        );
    }
    
    
    

    
    open(PIPE, "$cmd|");

    $BytesRead = 1;
    while($BytesRead != 0)
    {
        $var = undef;
        $ReadVar = undef;
        
        until( (ord($ReadVar) == 13) || ($BytesRead == 0) )
        {
            $BytesRead = read(PIPE,$ReadVar,1);
            $var = $var . $ReadVar;
        }
        
        $var =~ s/^\s+//;
        $var =~ s/\s+$//;
            
        unless($var == 0)
        {
            ($PercentDone) = split(/\%/,$var);
        }
        
        if($BytesRead == 0)
        {
            $var = "Complete";
        }
        
        $Install_Progress->configure(-to=>100);
        $Install_Progress->value($PercentDone);
        
        $Install_Label->configure(-text=>$var);
        $main->update;
    }
    &Attach
}

#################################################################################
sub Cancel
# onclick action of the Cancel_Button
#
# Prompts the user to cancel the install, then cancels if yes is pressed.
#################################################################################
{
    $Cancel_Result = $main->messageBox
    (
        -type=>"yesno",
       	-message=>"Are you sure?",
       	-title=>'Cancel Installation',
        -font=>"$font 10"
    );
    if($Cancel_Result eq "Yes")
    {
        close(PIPE);
        $Cancel_Flag = 1;
    }
}

#################################################################################
sub CleanUp
#
# Save configuration data on exit
#################################################################################
{
    &UpdateConfig;
    
    Tk::exit;
}

#################################################################################
sub CreateConfig
#
# Creates a new configuration file with default data.
#################################################################################
{
    open(CONFIG,">hdl_conf");
    &ConfigHeaders;
    print CONFIG "hdl_dump_location= ..\n";
    print CONFIG "\n";
    print CONFIG "ps2_ip= 192.168.0.10\n";
    close(CONFIG);
    
    $ps2_ip = '192.168.0.10';
    $Network_Text->configure(-text=>$ps2_ip);
    $hdl_dump = "./hdl_dump";
}

#################################################################################
sub UpdateConfig
#
# Updates the configuration file with data that was used this session.
#################################################################################
{
    open(CONFIG,">hdl_conf");
    &ConfigHeaders;
    if($hdl_dump eq "./hdl_dump")
    {
        print CONFIG "hdl_dump_location= ..\n";
    }
    else
    {
        print CONFIG "hdl_dump_location= $hdl_dump\n";
    }
    print CONFIG "\n";
    print CONFIG "ps2_ip= $ps2_ip\n";
    close(CONFIG);
}

#################################################################################
sub ConfigHeaders
#
# Headers to print to the configuration file.
#################################################################################
{
    print CONFIG "\#\n";
    print CONFIG "\# Config file for HDL_DUMB gui for linux\n";
    print CONFIG "\# Here you can modify the location of HDL_DUMP and the IP of your PS2\n";
    print CONFIG "\#\n";
    print CONFIG "\# For hdl_dump_location enter the full path of the hdl_dump execuable\n";
    print CONFIG "\# Ex. /usr/bin/hdl_dump\n";
    print CONFIG "\# The default is: \"..\" for the same directory hdl_dumb is located\n";
    print CONFIG "\n";
}

#################################################################################
sub ReadConfig
#
# Reads the configuration file, and places data where it needs to be.
#################################################################################
{
    open(CNFFILE, "<hdl_conf");
    while($line = <CNFFILE>)
    {
        next if($line =~ /^\s*#/);	# Skip Comment Lines
        $line = lc($line);
        if ( $line =~ /hdl_dump_location=/ )
        {
            $line =~ s/hdl_dump_location=//;
            $hdl_dump = &trim($line);
            
            if($hdl_dump =~ m/../)
            {
                $hdl_dump = "./hdl_dump";
            }
        }
        elsif ( $line =~ /ps2_ip=/ )
        {
            $line =~ s/ps2_ip=//;
            $ps2_ip = &trim($line);
        }
    }
    close(CNFFILE);
    # Make sure all the required options were found.
    if (!$ps2_ip)
    {
    # If not recreate config file
        &CreateConfig;
    }
    
    $Network_Text->configure(-text=>$ps2_ip);
    
}

############################################################################
sub trim {				
#
# Trims the data in the configuration file.
############################################################################

    my @out = @_;
    for (@out)
    {
        s/^\s+//;
	s/\s+$//;
    }
    return wantarray ? @out : $out[0];
}